@@ -4,7 +4,7 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = 0.1101;
+our $VERSION = 0.12;
 
 =head1 NAME
 
@@ -353,10 +353,9 @@ sub static {
 Mangle request before serving it.
 E.g. canonize uri or read session cookie.
 
-If the sub returns a MVC::Neaf::Request object in scalar context,
-it is considered to replace the original one.
-It looks like it's hard to return an unrelated Request by chance,
-but beware!
+Return value from callback is ignored.
+
+Dying in callback is treated the same way as in normal controller sub.
 
 =cut
 
@@ -537,50 +536,81 @@ sub set_session_handler {
     return $self;
 };
 
-=head2 server_stat ( MVC::Neaf::X::ServerStat->new( ... ) )
+=head2 set_error_handler ( status => CODEREF( $req, %options ) )
 
-Record server performance statistics during run.
+Set custom error handler.
 
-The interface of ServerStat is as follows:
+Status must be either a 3-digit number (as in HTTP), or "view".
+Other allowed keys MAY appear in the future.
 
-    my $stat = MVC::Neaf::X::ServerStat->new (
-        write_threshold_count => 100,
-        write_threshold_time  => 1,
-        on_write => sub {
-            my $array_of_arrays = shift;
+The following options will be passed to coderef:
 
-            foreach (@$array_of_arrays) {
-                # @$_ = (script_name, http_status,
-                #       controller_duration, total_duration, start_time)
-                # do something with this data
-                warn "$_->[0] returned $_->[1] in $_->[3] sec\n";
-            };
-        },
-    );
+=over
 
-on_write will be executed as soon as either count data points are accumulated,
-or time is exceeded by difference between first and last request in batch.
+=item * status - status being returned (500 in case of 'view');
 
-Returns self.
+=item * caller - array with the point where MVC::Neaf->route was set up;
+
+=item * error - exception, if there was one.
+
+=back
+
+The coderef MUST return an unblessed hash just like controller does.
+
+In case of exception or unexpected return format text message "Error XXX"
+will be returned instead.
+
+=head2 set_error_handler ( status => \%hash )
+
+Return a static template as { %options, %hash }.
 
 =cut
 
-sub server_stat {
-    my ($self, $obj) = @_;
+sub set_error_handler {
+    my ($self, $status, $code) = @_;
     $self = $Inst unless ref $self;
 
-    if ($obj) {
-        $self->{stat} = $obj;
-    } else {
-        delete $self->{stat};
+    $status =~ /^(\d\d\d|view)$/
+        or $self->_croak( "1st arg must be http status or a const(see docs)");
+    if (ref $code eq 'HASH') {
+        my $hash = $code;
+        $code = sub {
+            my ($req, %opt) = @_;
+
+            return { -status => $opt{status}, %opt, %$hash };
+        };
     };
+    ref $code eq 'CODE'
+        or $self->_croak( "2nd arg must be callback or hash");
+
+    $self->{error_template}{$status} = $code;
 
     return $self;
 };
 
+=head2 error_template( ... )
+
+B<DEPRECATED>. Same as above, but issues a warning.
+
+=cut
+
+sub error_template {
+    my $self = shift;
+
+    carp "error_template() is deprecated, use set_error_handler() instead";
+    return $self->set_error_handler(@_);
+};
+
 =head2 on_error( sub { my ($req, $err) = @_ } )
 
-Install custom error handler (e.g. write to log).
+Install custom error handler for dying controller.
+Neaf's own exceptions and 'die \d\d\d' status returns will NOT
+trigger it.
+
+E.g. write to log, or something.
+
+Return value from this callback is ignored.
+If it dies, only a warning is emitted.
 
 =cut
 
@@ -599,45 +629,6 @@ sub on_error {
     return $self;
 };
 
-=head2 error_template ( status => { ... } )
-
-Set custom error handler.
-
-Status must be either a 3-digit number (as in HTTP), or "view".
-Other allowed keys MAY appear in the future.
-
-The second argument follows the same rules as controller return.
-
-The following fields are expected to exist if this handler fires:
-
-=over
-
-=item * -status - defaults to actual status (e.g. 500);
-
-=item * caller - array with the point where MVC::Neaf->route was set up;
-
-=item * error - in case of exception, this will be the error.
-
-=back
-
-Returns self.
-
-=cut
-
-sub error_template {
-    my ($self, $status, $tpl) = @_;
-    $self = $Inst unless ref $self;
-
-    $status =~ /^(\d\d\d|view)$/
-        or $self->_croak( "1st arg must be http status or a const(see docs)");
-    ref $tpl eq 'HASH'
-        or $self->_croak( "2nd arg must be hash (just as controller returns)");
-
-    $self->{error_template}{$status} = $tpl;
-
-    return $self;
-};
-
 =head2 run()
 
 Run the applicaton.
@@ -775,6 +766,47 @@ sub set_forced_view {
     return $self;
 };
 
+=head2 server_stat ( MVC::Neaf::X::ServerStat->new( ... ) )
+
+Record server performance statistics during run.
+
+The interface of ServerStat is as follows:
+
+    my $stat = MVC::Neaf::X::ServerStat->new (
+        write_threshold_count => 100,
+        write_threshold_time  => 1,
+        on_write => sub {
+            my $array_of_arrays = shift;
+
+            foreach (@$array_of_arrays) {
+                # @$_ = (script_name, http_status,
+                #       controller_duration, total_duration, start_time)
+                # do something with this data
+                warn "$_->[0] returned $_->[1] in $_->[3] sec\n";
+            };
+        },
+    );
+
+on_write will be executed as soon as either count data points are accumulated,
+or time is exceeded by difference between first and last request in batch.
+
+Returns self.
+
+=cut
+
+sub server_stat {
+    my ($self, $obj) = @_;
+    $self = $Inst unless ref $self;
+
+    if ($obj) {
+        $self->{stat} = $obj;
+    } else {
+        delete $self->{stat};
+    };
+
+    return $self;
+};
+
 =head1 INTERNAL API
 
 B<CAVEAT EMPTOR.>
@@ -843,9 +875,7 @@ sub handle_request {
 
         # Try running the pre-routing callback.
         if (exists $self->{pre_route}) {
-            my $new_req = $self->{pre_route}->( $req );
-            blessed $new_req and $new_req->isa("MVC::Neaf::Request")
-                and $req = $new_req;
+            $self->{pre_route}->( $req );
         };
 
         # Lookup the rules for the given path
@@ -955,8 +985,9 @@ sub _error_to_reply {
         return $err;
     };
 
-    # A generic error...
     my $status = (!ref $err && $err =~ /^(\d\d\d)/) ? $1 : 500;
+
+    # Try exception handler
     if( !$1 ) {
         exists $self->{on_error}
             and eval { $self->{on_error}->($req, $err, $where) };
@@ -965,18 +996,26 @@ sub _error_to_reply {
             if $@;
     };
 
+    # Try fancy error template
     if (exists $self->{error_template}{$status}) {
-        my %ret = %{ $self->{error_template}{$status} }; # don't spoil the original
-        $ret{-status} = $status;
-        $ret{caller} = $where;
-        $ret{error} = $err;
-        return \%ret;
-    } else {
-        return {
-            -status     => $status,
-            -type       => 'text/plain',
-            -content    => "Error $status\n",
+        my $ret = eval {
+            $self->{error_template}{$status}->( $req,
+                status => $status,
+                caller => $where,
+                error => $err,
+            );
         };
+        return $ret
+            if (ref $ret eq 'HASH');
+        warn "ERROR: Error status handler for $status failed: $@"
+            if $@;
+    };
+
+    # Options exhausted - return plain error message
+    return {
+        -status     => $status,
+        -type       => 'text/plain',
+        -content    => "Error $status\n",
     };
 };
 
