@@ -3,11 +3,11 @@ package MVC::Neaf::Request;
 use strict;
 use warnings;
 
-our $VERSION = 0.1101;
+our $VERSION = 0.12;
 
 =head1 NAME
 
-MVC::Neaf::Request - Request class for Neaf.
+MVC::Neaf::Request - Request class for Not Even A Framework.
 
 =head1 DESCRIPTION
 
@@ -29,8 +29,8 @@ Here's a brief overview of what a Neaf request returns:
     $req->port         = 1337
     $req->path         = /mathing/route/some/more/slashes
     $req->script_name  = /mathing/route
-    $req->path_info    = /some/more/slashes
 
+    $req->path_info( '.+' )     = some/more/slashes
     $req->param( foo => '\d+' ) = 1
 
 =head1 REQUEST METHODS
@@ -212,23 +212,34 @@ sub script_name {
     return $self->{script_name};
 };
 
-=head2 path_info( [ $trim = 0|1 ])
+=head2 path_info( $regexp )
 
-Return path_info, the part of URL between script_name and parameters.
-Empty if no such part exists.
+Returns the part of URI path beyond what matched the application's path.
 
-If trim=1 is given, removed the leading slashes.
+Contrary to the
+L<CGI specification|https://tools.ietf.org/html/rfc3875#section-4.1.5>,
+the leading slash is REMOVED before checking the regexp.
+
+The WHOLE path_info value is matched against the given regular expression.
+
+If there's no match, undef is returned.
+
+Otherwise, $1 (if present) or the whole string is returned.
 
 =cut
 
 sub path_info {
-    my ($self, $trim) = @_;
+    my ($self, $regexp) = @_;
 
+    if (!defined $regexp) {
+        carp((ref $self)."->path_info(): DEPRECATED: validation regex is now REQUIRED");
+        $regexp = '.*';
+    };
     return '' unless exists $self->{path_info};
 
-    my $path = $self->{path_info};
-    $path =~ s#^/+## if $trim;
-    return $path;
+    $self->{path_info} =~ /^$regexp$/
+        or return undef; ## no critic # YES, return undef in list context
+    return defined $1 ? $1 : $self->{path_info};
 };
 
 =head2 set_full_path( $path )
@@ -259,15 +270,17 @@ sub set_full_path {
 
     if (defined $path_info) {
         # Make sure path_info always has a slash if nonempty
-        $path_info = "/$path_info" if $path_info =~ /^[^\/]/;
+        $path_info =~ s#^/+##;
         $self->{path_info} = Encode::is_utf8($path_info)
                 ? $path_info
                 : decode_utf8(uri_unescape($path_info));
     } elsif (!defined $self->{path_info}) {
         $self->{path_info} = '';
     };
+    # assert $self->{path_info} is defined by now
 
-    $self->{path} = "$self->{script_name}$self->{path_info}";
+    $self->{path} = "$self->{script_name}"
+        .(length $self->{path_info} ? "/$self->{path_info}" : '');
     return $self;
 };
 
@@ -286,23 +299,32 @@ sub set_path_info {
     my ($self, $path_info) = @_;
 
     $path_info = '' unless defined $path_info;
-    $path_info = "/$path_info" if $path_info =~ /^[^\/]/;
+    $path_info =~ s#^/+##;
 
     $self->{path_info} = $path_info;
+    $self->{path} = "$self->{script_name}"
+        .(length $self->{path_info} ? "/$self->{path_info}" : '');
 
-    $self->{path} = "$self->{script_name}$self->{path_info}";
     return $self;
 };
 
 =head2 param($name, $regex [, $default])
 
-Return param, if it passes regex check, default value (or '') otherwise.
+Return param, if it passes regex check, default value or undef otherwise.
 
 The regular expression is applied to the WHOLE string,
 from beginning to end, not just the middle.
 Use '.*' if you really need none.
 
-A default value of C<undef> is possible, but must be supplied explicitly.
+If method other than GET/HEAD is being used, whatever is in the
+address line after ? is IGNORED.
+Use url_param() (see below) if you intend to mix GET/POST parameters.
+
+B<NOTE> param() ALWAYS returns a single value, even in list context.
+Use multi_param() (see below) if you really want a list.
+
+B<NOTE> Behaviour changed since 0.11 - missing default value no more
+interpreted as '', returns undef.
 
 =cut
 
@@ -311,16 +333,86 @@ sub param {
 
     $self->_croak( "validation regex is REQUIRED" )
         unless defined $regex;
-    $default = '' if @_ <= 3; # deliberate undef as default = ok
 
     # Some write-through caching
     my $value = $self->_all_params->{ $name };
 
-    return (defined $value and $value =~ /^$regex$/s)
+    return (defined $value and $value =~ /^(?:$regex)$/s)
         ? $value
         : $default;
 };
 
+=head2 url_param( name => qr/regex/ )
+
+If method is GET or HEAD, identic to param.
+
+Otherwise would return the parameter from query string,
+AS IF it was a GET request.
+
+Multiple values are deliberately ignored.
+
+See L<CGI>.
+
+=cut
+
+our %query_allowed = ( GET => 1, HEAD => 1);
+sub url_param {
+    my ($self, $name, $regex, $default) = @_;
+
+    if ($query_allowed{ $self->method }) {
+        return $self->param( $name, $regex, $default );
+    };
+
+    # HACK here - some lazy caching + parsing string by hand
+    $self->{url_param_hash} ||= do {
+        my %hash;
+
+        foreach (split /[&;]/, $self->{query_string} || '' ) {
+            /^(.*?)(?:=(.*))?$/ or next;
+            $hash{$1} = $2;
+        };
+
+        # causes error w/o + (context issues?)
+        # do decoding AFTER uniq'ing params (plus it was simpler to write)
+        +{ map { decode_utf8(uri_unescape($_)) } %hash };
+    };
+    my $value = $self->{url_param_hash}{$name};
+
+    # this is copypaste from param(), do something (or don't)
+    return (defined $value and $value =~ /^(?:$regex)$/s)
+        ? $value
+        : $default;
+};
+
+=head2 multi_param( name => qr/regex/ )
+
+Get a single multivalue GET/POST parameter as a @list.
+The name generally follows that of newer L<CGI> (4.08+).
+
+ALL values must match the regex, or an empty list is returned.
+
+B<EXPERIMENTAL> This method's behaviour MAY change in the future.
+Please be careful when upgrading.
+
+=cut
+
+# TODO merge multi_param, param, and _all_params
+# backend mechanism.
+
+sub multi_param {
+    my ($self, $name, $regex) = @_;
+
+    $self->_croak( "validation regex is REQUIRED" )
+        unless defined $regex;
+
+    my $ret = $self->{multi_param}{$name} ||= [
+        map { decode_utf8($_) } $self->do_get_param_as_array( $name ),
+    ];
+
+    # ANY mismatch = no go. Replace with simple grep if want filter EVER.
+    return (grep { !/^(?:$regex)$/s } @$ret) ? () : @$ret;
+};
+
 =head2 set_param( name => $value )
 
 Override form parameter. Returns self.
@@ -420,6 +512,23 @@ sub _all_params {
     };
 };
 
+=head2 body()
+
+Returns request body for PUT/POST requests.
+This is not regex-checked - the check is left for the user.
+
+Also the data is NOT converted to utf8.
+
+=cut
+
+sub body {
+    my $self = shift;
+
+    $self->{body} = $self->do_get_body
+        unless exists $self->{body};
+    return $self->{body};
+};
+
 =head2 set_default( key => $value, ... )
 
 Set default values for your return hash.
@@ -753,7 +862,7 @@ sub dump {
 
     my %raw;
     foreach my $method (qw( http_version scheme secure method hostname port
-        path script_name path_info
+        path script_name
         referer user_agent )) {
             $raw{$method} = eval { $self->$method }; # deliberately skip errors
     };
@@ -761,6 +870,7 @@ sub dump {
     $raw{header_in} = $self->header_in->as_string;
     $self->get_cookie( noexist => '' ); # warm up cookie cache
     $raw{cookie_in} = $self->{neaf_cookie_in};
+    $raw{path_info} = $self->path_info( qr'.*'s );
 
     return \%raw;
 };
@@ -786,25 +896,55 @@ B<NOTE> currently session-related methods don't die if session handler
 wasn't set. Instead, they behave as if session wasn't there.
 This MAY change in the future.
 
+B<NOTE> The underlying API was changed in 0.12 (see L<MVC::Neaf::X::Session>).
+The fallback & warn code will stay until 0.15, after that older
+session implementations MUST be rewritten.
+
 =cut
 
+my %known_session_keys = ( data => 1, id => 1, expire => 1 ); # TODO Remove 0.15
+
 sub session {
     my ($self, $noinit) = @_;
 
     # agressive caching FTW
     return $self->{session} if exists $self->{session};
 
-    if ($self->{session_engine}) {
-        my $id = $self->get_cookie( $self->{session_cookie}, $self->{session_regex} );
-        $self->{session} = $self->{session_engine}->load_session( $id )
-            if $id;
-        $self->{session} ||= $self->{session_engine}->create_session
-            unless $noinit;
-    } else {
-        # TODO should we die if no engine?..
-        $self->{session} = {}
-            unless $noinit;
+    if (!$self->{session_engine}) {
+        # TODO should we just die here?
+        $self->{session} ||= {} unless $noinit;
+        return $self->{session};
     };
+
+    # Try loading session...
+    my $id = $self->get_cookie( $self->{session_cookie}, $self->{session_regex} );
+    my $hash = ($id && $self->{session_engine}->load_session( $id ));
+
+    # TODO remove the below block in 0.15 - deprecated API warning
+    if ($hash && ref $hash eq 'HASH') {
+        foreach (keys %$hash) {
+            $known_session_keys{ $_ } and next;
+            carp "DEPRECATED load_session/save_session API changed in Neaf 0.12, trying to continue";
+            $hash = { data => $hash };
+            last;
+        };
+    } elsif ($hash && ref $hash ne 'HASH') {
+        carp "DEPRECATED load_session/save_session API changed in Neaf 0.12, trying to continue";
+        $hash = { data => $hash };
+    };
+
+    if ($hash && ref $hash eq 'HASH' && $hash->{data}) {
+        # Loaded, cache it & refresh if needed
+        $self->{session} = $hash->{data};
+
+        $self->set_cookie(
+            $self->{session_cookie} => $hash->{id}, expire => $hash->{expire} )
+                if $hash->{id};
+    } elsif ( !$noinit ) {
+        # Not loaded - init
+        $self->{session} = $self->{session_engine}->create_session
+    };
+
     return $self->{session};
 };
 
@@ -829,14 +969,34 @@ sub save_session {
 
     # TODO set "save session" flag, save later
     my $id = $self->get_cookie( $self->{session_cookie}, $self->{session_regex} );
+    $id ||= $self->{session_engine}->get_session_id();
 
-    if (!$id) {
-        # new session - generate id!
-        $id = $self->{session_engine}->get_session_id();
+    my $hash = $self->{session_engine}->save_session( $id, $self->session );
+
+    # TODO remove the below block in 0.15 - deprecated API warning
+    if ($hash && ref $hash eq 'HASH') {
+        foreach (keys %$hash) {
+            $known_session_keys{ $_ } and next;
+            carp "DEPRECATED load_session/save_session API changed in Neaf 0.12, trying to continue";
+            $hash = { id => $id };
+            last;
+        };
+    } elsif ($hash && ref $hash ne 'HASH') {
+        carp "DEPRECATED load_session/save_session API changed in Neaf 0.12, trying to continue";
+        $hash = { id => $id };
+    };
+
+    if ( $hash && ref $hash eq 'HASH' && $hash->{id} ) {
+        # save successful - send cookie to user
+        my $expire = $hash->{expire};
+        if (!defined $expire
+            and defined (my $ttl = $self->{session_engine}->session_ttl)
+        ) {
+            $expire = time + $ttl;
+        };
+        $self->set_cookie( $self->{session_cookie} => $hash->{id}, expire => $expire );
     };
 
-    $self->set_cookie( $self->{session_cookie} => $id, ttl => $self->{session_ttl} );
-    $self->{session_engine}->save_session( $id, $self->session );
     return $self;
 };
 
@@ -1075,8 +1235,12 @@ They shall not generally be called directly inside the app.
 
 =item * do_get_params()
 
+=item * do_get_param_as_array() - get single GET/POST param in list context
+
 =item * do_get_upload()
 
+=item * do_get_body()
+
 =item * do_get_header_in() - returns a HTTP::Headers object.
 
 =item * do_reply( $status, $content ) - write reply to client
@@ -1092,7 +1256,8 @@ They shall not generally be called directly inside the app.
 foreach (qw(
     do_get_method do_get_scheme do_get_hostname do_get_port do_get_path
     do_get_client_ip do_get_http_version
-    do_get_params do_get_upload do_get_header_in
+    do_get_params do_get_param_as_array do_get_upload do_get_header_in
+    do_get_body
     do_reply do_write)) {
     my $method = $_;
     my $code = sub {
