@@ -3,11 +3,11 @@ package MVC::Neaf::Request::Apache2;
 use strict;
 use warnings;
 
-our $VERSION = 0.11;
+our $VERSION = 0.12;
 
 =head1 NAME
 
-MVC::Neaf::Request::Apache - Apache2 (mod_perl) driver for Not Even A Framework.
+MVC::Neaf::Request::Apache2 - Apache2 (mod_perl) driver for Not Even A Framework.
 
 =head1 DESCRIPTION
 
@@ -158,6 +158,16 @@ sub do_get_params {
     return \%hash;
 };
 
+=head2 do_get_param_as_array
+
+=cut
+
+sub do_get_param_as_array {
+    my ($self, $name) = @_;
+
+    return $self->{driver}->param( $name );
+};
+
 =head2 do_get_header_in()
 
 =cut
@@ -193,6 +203,25 @@ sub do_get_upload {
     } : ();
 };
 
+=head2 do_get_body
+
+=cut
+
+sub do_get_body {
+    my $self = shift;
+
+    # use Apache2::RequestIO
+    # read until there's EOF, then concatenate & return
+    my $r = $self->{driver_raw};
+
+    my @buf = ('');
+    while ( $r->read( $buf[-1], 8192, 0 ) ) {
+        push @buf, '';
+    };
+
+    return join '', @buf;
+};
+
 =head2 do_reply( $status, $content )
 
 =cut
@@ -243,7 +272,11 @@ sub handler : method {
     my $self = $class->new(
         driver_raw => $r,
         driver => Apache2::Request->new($r),
+        query_string => $r->args,
     );
+    if (!$MVC::Neaf::Request::query_allowed{ $r->method }) {
+        $r->args('');
+    };
     my $reply = MVC::Neaf->handle_request( $self );
 
     return Apache2::Const::OK();
