@@ -2,7 +2,7 @@ package MVC::Neaf::Request::CGI;
 
 use strict;
 use warnings;
-our $VERSION = 0.11;
+our $VERSION = 0.12;
 
 =head1 NAME
 
@@ -44,6 +44,8 @@ sub new {
 
     $args{driver} ||= $cgi->new;
     $args{fd}     ||= \*STDOUT;
+    $args{query_string} ||= $1
+        if( ($args{driver}->request_uri || '') =~ m#.*?\?(.+)# );
 
     return bless \%args, $class;
 };
@@ -133,6 +135,16 @@ sub do_get_params {
     return \%hash;
 };
 
+=head2 do_get_param_as_array
+
+=cut
+
+sub do_get_param_as_array {
+    my ($self, $name) = @_;
+
+    return $self->{driver}->param( $name );
+};
+
 =head2 do_get_path
 
 =cut
@@ -176,6 +188,23 @@ sub do_get_header_in {
     return $head;
 };
 
+=head2 do_get_body()
+
+=cut
+
+sub do_get_body {
+    my $self = shift;
+
+    # TODO also check content-type header
+    if ($self->method eq 'POST') {
+        return $self->{driver}->param('POSTDATA');
+    } elsif ($self->method eq 'PUT') {
+        return $self->{driver}->param('PUTDATA');
+    };
+
+    return;
+};
+
 =head2 do_reply( $status, $content )
 
 =cut
