@@ -2,7 +2,7 @@ package MVC::Neaf::Request::PSGI;
 
 use strict;
 use warnings;
-our $VERSION = 0.11;
+our $VERSION = 0.12;
 
 =head1 NAME
 
@@ -20,14 +20,32 @@ use parent qw(MVC::Neaf::Request);
 
 =head2 new( env => $psgi_input )
 
-Constructor.
+Constructor. C<env> MUST follow L<PSGI> requirements.
 
 =cut
 
+my %default_env = (
+    REQUEST_METHOD => 'GET',
+);
+
 sub new {
     my $class = shift;
+
     my $self = $class->SUPER::new( @_ );
-    $self->{driver} ||= Plack::Request->new( $self->{env} || {} );
+
+    # Don't modify env!
+    # Remove query string if not GET|HEAD
+    # so that GET params are not available inside POST by default
+    my $env = $self->{env} || \%default_env;
+    $self->{query_string} = $env->{QUERY_STRING};
+
+    $self->{driver} ||= Plack::Request->new({
+        REQUEST_METHOD => 'GET',
+        %$env,
+        ($MVC::Neaf::Request::query_allowed{ $env->{REQUEST_METHOD} || 'GET' }
+            ? () : (QUERY_STRING => '')),
+    });
+
     return $self;
 };
 
@@ -133,6 +151,16 @@ sub do_get_params {
     return \%hash;
 };
 
+=head2 do_get_param_as_array
+
+=cut
+
+sub do_get_param_as_array {
+    my ($self, $name) = @_;
+
+    return $self->{driver}->param( $name );
+};
+
 =head2 do_get_upload( "name" )
 
 B<NOTE> This garbles Hash::Multivalue.
@@ -158,6 +186,16 @@ sub do_get_header_in {
     return $self->{driver}->headers;
 };
 
+=head2 do_get_body
+
+=cut
+
+sub do_get_body {
+    my $self = shift;
+
+    return $self->{driver}->content;
+};
+
 =head2 do_reply( $status_line, \%headers, $content )
 
 Send reply to client. Not to be used directly.
