@@ -2,7 +2,7 @@ package MVC::Neaf::X::Session::File;
 
 use strict;
 use warnings;
-our $VERSION = '0.2800_01';
+our $VERSION = 0.12;
 
 =head1 NAME
 
@@ -20,9 +20,6 @@ Uses flock() to avoid collisions.
 
 If session_ttl was specified, old session files will be deleted.
 
-B<NOTE> The file-locking MAY be prone to race conditions. If you want real secure
-expiration, please specify expiration INSIDE the session, or use a database.
-
 =head1 SYNOPSIS
 
     use strict;
@@ -40,9 +37,9 @@ expiration, please specify expiration INSIDE the session, or use a database.
 =cut
 
 use Fcntl qw(:flock :seek);
+use JSON::XS;
 use URI::Escape qw(uri_escape);
 
-use MVC::Neaf::Util qw(JSON encode_json decode_json);
 use parent qw(MVC::Neaf::X::Session);
 
 =head2 new( %options )
@@ -114,7 +111,7 @@ sub delete_session {
     my ($self, $id) = @_;
 
     if (!unlink $self->get_file_name( $id )) {
-        return 0 if $!{ENOENT} or $!{EPERM} && $^O eq 'MSWin32'; # missing = ok, locked+mswin = ok
+        return 0 if $!{ENOENT}; # missing = ok
         $self->my_croak( "Failed to delete file ".($self->get_file_name( $id ))
             .": $!" );
     };
@@ -144,7 +141,6 @@ sub atomic_read {
     my $ttl = $self->session_ttl;
     my $expire = $ttl && [stat $fd]->[9] + $ttl;
     if ($expire && $expire < time) {
-        close $fd if $^O eq 'MSWin32'; # won't delete under windows
         $self->delete_session( $id );
         return;
     };
@@ -220,18 +216,4 @@ sub decode_content {
     return decode_json( $raw );
 };
 
-=head1 LICENSE AND COPYRIGHT
-
-This module is part of L<MVC::Neaf> suite.
-
-Copyright 2016-2023 Konstantin S. Uvarin C<khedin@cpan.org>.
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See L<http://dev.perl.org/licenses/> for more information.
-
-=cut
-
 1;
